@@ -1,35 +1,54 @@
 """
 Core implementation of :mod:`sklearndf`
 """
-
+import inspect
 import logging
 from abc import ABCMeta, abstractmethod
-from typing import Any, List, Mapping, Optional, Sequence, TypeVar, Union, cast
+from typing import Any, Dict, List, Mapping, Optional, TypeVar, Union, cast
 
+import numpy.typing as npt
 import pandas as pd
 from sklearn.base import (
     BaseEstimator,
     ClassifierMixin,
+    ClusterMixin,
     RegressorMixin,
     TransformerMixin,
     clone,
 )
+from sklearn.exceptions import NotFittedError
+from sklearn.utils import is_scalar_nan
 
-from pytools.api import AllTracker
-from pytools.fit import FittableMixin
+from pytools.api import AllTracker, inheritdoc
+from pytools.expression import Expression, HasExpressionRepr, make_expression
+from pytools.expression.atomic import Id
 
 log = logging.getLogger(__name__)
 
-__all__ = ["EstimatorDF", "LearnerDF", "ClassifierDF", "RegressorDF", "TransformerDF"]
+__all__ = [
+    "ClassifierDF",
+    "ClusterDF",
+    "EstimatorDF",
+    "LearnerDF",
+    "RegressorDF",
+    "SupervisedLearnerDF",
+    "TransformerDF",
+]
 
 #
 # type variables
 #
 
-T_Self = TypeVar("T_Self")
 T_EstimatorDF = TypeVar("T_EstimatorDF")
 
 
+#
+# Constants
+#
+
+UNDEFINED = object()
+
+
 #
 # Ensure all symbols introduced below are included in __all__
 #
@@ -42,9 +61,14 @@ __tracker = AllTracker(globals())
 #
 
 
-class EstimatorDF(BaseEstimator, FittableMixin[pd.DataFrame], metaclass=ABCMeta):
+@inheritdoc(match="""[see superclass]""")
+class EstimatorDF(
+    HasExpressionRepr,
+    BaseEstimator,  # type: ignore
+    metaclass=ABCMeta,
+):
     """
-    Base class for augmented scikit-learn `estimators`.
+    Base class for augmented `scikit-learn` estimators.
 
     Provides enhanced support for data frames.
     """
@@ -70,11 +94,11 @@ class EstimatorDF(BaseEstimator, FittableMixin[pd.DataFrame], metaclass=ABCMeta)
     # noinspection PyPep8Naming
     @abstractmethod
     def fit(
-        self: T_Self,
+        self: T_EstimatorDF,
         X: pd.DataFrame,
         y: Optional[Union[pd.Series, pd.DataFrame]] = None,
         **fit_params: Any,
-    ) -> T_Self:
+    ) -> T_EstimatorDF:
         """
         Fit this estimator using the given inputs.
 
@@ -86,6 +110,24 @@ class EstimatorDF(BaseEstimator, FittableMixin[pd.DataFrame], metaclass=ABCMeta)
         """
         pass
 
+    @property
+    @abstractmethod
+    def is_fitted(self) -> bool:
+        """
+        ``True`` if this object is fitted, ``False`` otherwise.
+        """
+        pass
+
+    def ensure_fitted(self) -> None:
+        """
+        Raise a :class:`~sklearn.exceptions.NotFittedError` if this estimator is not
+        fitted.
+
+        :raise sklearn.exceptions.NotFittedError: this estimator is not fitted
+        """
+        if not self.is_fitted:
+            raise NotFittedError(f"{type(self).__name__} is not fitted")
+
     @property
     def feature_names_in_(self) -> pd.Index:
         """
@@ -94,7 +136,7 @@ class EstimatorDF(BaseEstimator, FittableMixin[pd.DataFrame], metaclass=ABCMeta)
 
         :raises AttributeError: if this estimator is not fitted
         """
-        self._ensure_fitted()
+        self.ensure_fitted()
         return self._get_features_in().rename(self.COL_FEATURE_IN)
 
     @property
@@ -104,7 +146,7 @@ class EstimatorDF(BaseEstimator, FittableMixin[pd.DataFrame], metaclass=ABCMeta)
 
         :raises AttributeError: if this estimator is not fitted
         """
-        self._ensure_fitted()
+        self.ensure_fitted()
         return self._get_n_outputs()
 
     def get_params(self, deep: bool = True) -> Mapping[str, Any]:
@@ -116,9 +158,9 @@ class EstimatorDF(BaseEstimator, FittableMixin[pd.DataFrame], metaclass=ABCMeta)
         :return: a mapping of parameter names to their values
         """
         # noinspection PyUnresolvedReferences
-        return super().get_params(deep=deep)
+        return cast(Mapping[str, Any], super().get_params(deep=deep))
 
-    def set_params(self: T_Self, **params: Any) -> T_Self:
+    def set_params(self: T_EstimatorDF, **params: Any) -> T_EstimatorDF:
         """
         Set the parameters of this estimator.
 
@@ -128,7 +170,7 @@ class EstimatorDF(BaseEstimator, FittableMixin[pd.DataFrame], metaclass=ABCMeta)
         :return: ``self``
         """
         # noinspection PyUnresolvedReferences
-        return super().set_params(**params)
+        return super().set_params(**params)  # type: ignore
 
     def clone(self: T_EstimatorDF) -> T_EstimatorDF:
         """
@@ -136,7 +178,7 @@ class EstimatorDF(BaseEstimator, FittableMixin[pd.DataFrame], metaclass=ABCMeta)
 
         :return: the unfitted clone
         """
-        return clone(self)
+        return cast(T_EstimatorDF, clone(self))
 
     @abstractmethod
     def _get_features_in(self) -> pd.Index:
@@ -148,10 +190,81 @@ class EstimatorDF(BaseEstimator, FittableMixin[pd.DataFrame], metaclass=ABCMeta)
         # get the number of outputs this estimator has been fitted to
         pass
 
+    def to_expression(self) -> Expression:
+        """[see superclass]"""
+
+        # create a dictionary with all parameters of the estimator, mapping them
+        # to their default values if defined (and otherwise to Signature.empty)
+        estimator_parameters = {
+            name: parameter.default
+            for name, parameter in inspect.signature(
+                self.__init__  # type: ignore
+            ).parameters.items()
+        }
+
+        def _kwarg_to_expression(name: str, value: Any) -> Optional[Expression]:
+            expression = make_expression(value)
+
+            # are we dealing with a default value?
+            default_value = estimator_parameters.get(name, UNDEFINED)
+
+            if (
+                (
+                    # there is a parameter with the given name
+                    default_value
+                    is not UNDEFINED
+                )
+                and (
+                    # the parameter has a default value
+                    default_value
+                    != inspect.Signature.empty
+                )
+                and (
+                    # if the value is an estimator ...
+                    not isinstance(value, BaseEstimator)
+                    # ... then it has the same class as the default value
+                    or value.__class__ == default_value.__class__
+                )
+                and (
+                    # both value and default value are np.nan ...
+                    (is_scalar_nan(value) and is_scalar_nan(default_value))
+                    or (
+                        # ... or both have the same expression.
+                        # We cannot compare for equality since we don't know
+                        # if the classes of the values implement this.
+                        # Therefore, we compare the expressions but do this last,
+                        # as it might be computationally more costly in the
+                        # unlikely case that the default value is a very complex
+                        # object.
+                        expression.eq_(make_expression(default_value))
+                    )
+                )
+            ):
+                # we can confirm the given value is the default value - no need
+                # to include in the expression representation of this estimator
+                return None
+
+            else:
+                # custom value: we return the expression
+                return expression
+
+        kwarg_expressions: Dict[str, Optional[Expression]] = {
+            name: _kwarg_to_expression(name, value)
+            for name, value in self.get_params(deep=False).items()
+        }
+
+        return Id(type(self))(
+            **{
+                name: expression
+                for name, expression in kwarg_expressions.items()
+                if expression is not None
+            }
+        )
+
 
 class LearnerDF(EstimatorDF, metaclass=ABCMeta):
     """
-    Base class for augmented scikit-learn `learners`.
+    Base class for augmented `scikit-learn` learners.
 
     Provides enhanced support for data frames.
     """
@@ -177,22 +290,13 @@ class LearnerDF(EstimatorDF, metaclass=ABCMeta):
         """
         pass
 
-    # noinspection PyPep8Naming
-    @abstractmethod
-    def fit_predict(
-        self, X: pd.DataFrame, y: pd.Series, **fit_params: Any
-    ) -> Union[pd.Series, pd.DataFrame]:
-        """
-        Fit this learner using the given inputs, then predict the outputs.
 
-        :param X: data frame with observations as rows and features as columns
-        :param y: a series or data frame with one or more outputs per observation
-        :param fit_params: optional keyword parameters as required by specific
-            learner implementations
-        :return: predictions per observation as a series, or as a data frame in case
-            of multiple outputs
-        """
-        pass
+class SupervisedLearnerDF(LearnerDF, metaclass=ABCMeta):
+    """
+    Base class for augmented `scikit-learn` supervised learners.
+
+    Provides enhanced support for data frames.
+    """
 
     # noinspection PyPep8Naming
     @abstractmethod
@@ -212,9 +316,13 @@ class LearnerDF(EstimatorDF, metaclass=ABCMeta):
         pass
 
 
-class TransformerDF(EstimatorDF, TransformerMixin, metaclass=ABCMeta):
+class TransformerDF(
+    TransformerMixin,  # type: ignore
+    EstimatorDF,
+    metaclass=ABCMeta,
+):
     """
-    Base class for augmented scikit-learn `transformers`.
+    Base class for augmented `scikit-learn` transformers.
 
     Provides enhanced support for data frames.
     """
@@ -236,6 +344,7 @@ class TransformerDF(EstimatorDF, TransformerMixin, metaclass=ABCMeta):
 
     @property
     def feature_names_original_(self) -> pd.Series:
+        # noinspection GrazieInspection
         """
         A pandas series, mapping the output features resulting from the transformation
         to the original input features.
@@ -243,7 +352,7 @@ class TransformerDF(EstimatorDF, TransformerMixin, metaclass=ABCMeta):
         The index of the resulting series consists of the names of the output features;
         the corresponding values are the names of the original input features.
         """
-        self._ensure_fitted()
+        self.ensure_fitted()
         if self._features_original is None:
             self._features_original = (
                 self._get_features_original()
@@ -258,7 +367,7 @@ class TransformerDF(EstimatorDF, TransformerMixin, metaclass=ABCMeta):
         A pandas column index with the names of the features produced by this
         transformer
         """
-        self._ensure_fitted()
+        self.ensure_fitted()
         return self._get_features_out().rename(self.COL_FEATURE_OUT)
 
     # noinspection PyPep8Naming
@@ -323,29 +432,48 @@ class TransformerDF(EstimatorDF, TransformerMixin, metaclass=ABCMeta):
         return self.feature_names_original_.index
 
 
-class RegressorDF(LearnerDF, RegressorMixin, metaclass=ABCMeta):
+class RegressorDF(
+    RegressorMixin,  # type: ignore
+    SupervisedLearnerDF,
+    metaclass=ABCMeta,
+):
     """
-    Base class for augmented scikit-learn `regressors`.
+    Base class for augmented `scikit-learn` regressors.
 
     Provides enhanced support for data frames.
     """
 
+    # noinspection PyPep8Naming
+    @abstractmethod
+    def score(
+        self, X: pd.DataFrame, y: pd.Series, sample_weight: Optional[pd.Series] = None
+    ) -> float:
+        """[see SupervisedLearnerDF]"""
 
-class ClassifierDF(LearnerDF, ClassifierMixin, metaclass=ABCMeta):
+    # we cannot get the docstring via the @inheritdoc mechanism because
+    # RegressorMixin precedes SupervisedLearnerDF in the MRO
+    score.__doc__ = SupervisedLearnerDF.score.__doc__
+
+
+class ClassifierDF(
+    ClassifierMixin,  # type: ignore
+    SupervisedLearnerDF,
+    metaclass=ABCMeta,
+):
     """
-    Base class for augmented scikit-learn `classifiers`.
+    Base class for augmented `scikit-learn` classifiers.
 
     Provides enhanced support for data frames.
     """
 
     @property
     @abstractmethod
-    def classes_(self) -> Sequence[Any]:
+    def classes_(self) -> Union[npt.NDArray[Any], List[npt.NDArray[Any]]]:
         """
         Get the classes predicted by this classifier.
-        By default expects classes as a list-like stored in the `classes_` attribute.
 
-        :return: the classes predicted by this classifier
+        :return: a numpy array of class labels for single-output problems, or a list
+            of such arrays for multi-output problems
         """
         pass
 
@@ -415,5 +543,57 @@ class ClassifierDF(LearnerDF, ClassifierMixin, metaclass=ABCMeta):
             per output
         """
 
+    # noinspection PyPep8Naming
+    @abstractmethod
+    def score(
+        self, X: pd.DataFrame, y: pd.Series, sample_weight: Optional[pd.Series] = None
+    ) -> float:
+        """[see SupervisedLearnerDF]"""
+
+    # we cannot get the docstring via the @inheritdoc mechanism because
+    # ClassifierMixin precedes SupervisedLearnerDF in the MRO
+    score.__doc__ = SupervisedLearnerDF.score.__doc__
+
+
+class ClusterDF(
+    ClusterMixin,  # type: ignore
+    LearnerDF,
+    metaclass=ABCMeta,
+):
+    """
+    Base class for augmented `scikit-learn` clusterers.
+
+    Provides enhanced support for data frames.
+    """
+
+    @property
+    @abstractmethod
+    def labels_(self) -> pd.Series:
+        # noinspection GrazieInspection
+        """
+        A pandas series, mapping the index of the input data frame to cluster labels.
+        """
+        pass
+
+    # noinspection PyPep8Naming
+    @abstractmethod
+    def fit_predict(
+        self,
+        X: pd.DataFrame,
+        y: Optional[Union[pd.Series, pd.DataFrame]] = None,
+        **fit_predict_params: Any,
+    ) -> Union[pd.Series, pd.DataFrame]:
+        """
+        Fit this clusterer using the given inputs, then predict the cluster labels.
+
+        :param X: data frame with observations as rows and features as columns
+        :param y: a series or data frame with one or more outputs per observation
+        :param fit_predict_params: optional keyword parameters as required by specific
+            clusterer implementations
+        :return: predicted cluster labels for all observations as a series,
+            or as a data frame in case of multiple outputs
+        """
+        pass
+
 
 __tracker.validate()
