@@ -1,16 +1,20 @@
 """
 Additional implementation of :mod:`sklearndf.classification` loaded
-from sklearn 0.23 onwards
+from sklearn 0.22 onwards
 """
-
 import logging
-from typing import List
+
+from sklearn.ensemble import StackingClassifier
+from sklearn.naive_bayes import CategoricalNB
 
 from pytools.api import AllTracker
 
+from ..wrapper.stacking import StackingClassifierWrapperDF
+from .wrapper import PartialFitClassifierWrapperDF
+
 log = logging.getLogger(__name__)
 
-__all__: List[str] = []
+__all__ = ["CategoricalNBDF", "StackingClassifierDF"]
 
 __imported_estimators = {name for name in globals().keys() if name.endswith("DF")}
 
@@ -27,11 +31,32 @@ __tracker = AllTracker(globals())
 #
 
 
-# todo: add classification implementations for sklearn 0.23
+#
+# naive bayes
+#
 
 
+class CategoricalNBDF(
+    PartialFitClassifierWrapperDF[CategoricalNB],
+    native=CategoricalNB,
+):
+    """Stub for DF wrapper of class ``CategoricalNB``"""
+
+
+class StackingClassifierDF(
+    StackingClassifierWrapperDF[StackingClassifier],
+    native=StackingClassifier,
+):
+    """Stub for DF wrapper of class ``StackingClassifier``"""
+
+
+#
+# validate __all__
+#
+
 __tracker.validate()
 
+
 #
 # validate that __all__ comprises all symbols ending in "DF", and no others
 #
