@@ -4,23 +4,44 @@ GAMMA custom two-step pipelines
 
 import logging
 from abc import ABCMeta, abstractmethod
-from typing import Any, Generic, List, Optional, Sequence, TypeVar, Union
+from typing import Any, Generic, List, Optional, TypeVar, Union
 
+import numpy.typing as npt
 import pandas as pd
 
 from pytools.api import AllTracker, inheritdoc
 
-from .. import ClassifierDF, EstimatorDF, LearnerDF, RegressorDF, TransformerDF
+from .. import (
+    ClassifierDF,
+    ClusterDF,
+    EstimatorDF,
+    LearnerDF,
+    RegressorDF,
+    SupervisedLearnerDF,
+    TransformerDF,
+)
 
 log = logging.getLogger(__name__)
 
-__all__ = ["LearnerPipelineDF", "RegressorPipelineDF", "ClassifierPipelineDF"]
+__all__ = [
+    "LearnerPipelineDF",
+    "SupervisedLearnerPipelineDF",
+    "RegressorPipelineDF",
+    "ClassifierPipelineDF",
+    "ClusterPipelineDF",
+]
 
-T_Self = TypeVar("T_Self")
+T_EstimatorPipelineDF = TypeVar(
+    "T_EstimatorPipelineDF", bound="_EstimatorPipelineDF[EstimatorDF]"
+)
 T_FinalEstimatorDF = TypeVar("T_FinalEstimatorDF", bound=EstimatorDF)
 T_FinalLearnerDF = TypeVar("T_FinalLearnerDF", bound=LearnerDF)
+T_FinalSupervisedLearnerDF = TypeVar(
+    "T_FinalSupervisedLearnerDF", bound=SupervisedLearnerDF
+)
 T_FinalRegressorDF = TypeVar("T_FinalRegressorDF", bound=RegressorDF)
 T_FinalClassifierDF = TypeVar("T_FinalClassifierDF", bound=ClassifierDF)
+T_FinalClusterDF = TypeVar("T_FinalClusterDF", bound=ClusterDF)
 
 
 #
@@ -117,13 +138,13 @@ class _EstimatorPipelineDF(EstimatorDF, Generic[T_FinalEstimatorDF], metaclass=A
 
     # noinspection PyPep8Naming
     def fit(
-        self: T_Self,
+        self: T_EstimatorPipelineDF,
         X: pd.DataFrame,
         y: Optional[Union[pd.Series, pd.DataFrame]] = None,
         *,
         sample_weight: Optional[pd.Series] = None,
-        **fit_params,
-    ) -> T_Self:
+        **fit_params: Any,
+    ) -> T_EstimatorPipelineDF:
         """
         Fit this pipeline using the given inputs.
 
@@ -135,15 +156,14 @@ class _EstimatorPipelineDF(EstimatorDF, Generic[T_FinalEstimatorDF], metaclass=A
             estimator implementations
         :return: ``self``
         """
-        self: _EstimatorPipelineDF  # support type hinting in PyCharm
 
-        X_preprocessed: pd.DataFrame = self._pre_fit_transform(X, y, **fit_params)
+        x_preprocessed: pd.DataFrame = self._pre_fit_transform(X, y, **fit_params)
 
         if sample_weight is None:
-            self.final_estimator.fit(X_preprocessed, y, **fit_params)
+            self.final_estimator.fit(x_preprocessed, y, **fit_params)
         else:
             self.final_estimator.fit(
-                X_preprocessed, y, sample_weight=sample_weight, **fit_params
+                x_preprocessed, y, sample_weight=sample_weight, **fit_params
             )
 
         return self
@@ -176,7 +196,7 @@ class _EstimatorPipelineDF(EstimatorDF, Generic[T_FinalEstimatorDF], metaclass=A
 
     # noinspection PyPep8Naming
     def _pre_fit_transform(
-        self, X: pd.DataFrame, y: pd.Series, **fit_params
+        self, X: pd.DataFrame, y: pd.Series, **fit_params: Any
     ) -> pd.DataFrame:
         if self.preprocessing is not None:
             return self.preprocessing.fit_transform(X, y, **fit_params)
@@ -203,14 +223,18 @@ class LearnerPipelineDF(
         """[see superclass]"""
         return self.final_estimator.predict(self._pre_transform(X), **predict_params)
 
-    # noinspection PyPep8Naming
-    def fit_predict(
-        self, X: pd.DataFrame, y: pd.Series, **fit_params: Any
-    ) -> Union[pd.Series, pd.DataFrame]:
-        """[see superclass]"""
-        return self.final_estimator.fit_predict(
-            self._pre_fit_transform(X, y, **fit_params), y, **fit_params
-        )
+
+@inheritdoc(match="[see superclass]")
+class SupervisedLearnerPipelineDF(
+    LearnerPipelineDF[T_FinalSupervisedLearnerDF],
+    SupervisedLearnerDF,
+    Generic[T_FinalSupervisedLearnerDF],
+    metaclass=ABCMeta,
+):
+    """
+    A data frame enabled pipeline with an optional preprocessing step and a
+    mandatory supervised learner step.
+    """
 
     # noinspection PyPep8Naming
     def score(
@@ -230,7 +254,9 @@ class LearnerPipelineDF(
 
 @inheritdoc(match="[see superclass]")
 class RegressorPipelineDF(
-    LearnerPipelineDF[T_FinalRegressorDF], RegressorDF, Generic[T_FinalRegressorDF]
+    SupervisedLearnerPipelineDF[T_FinalRegressorDF],
+    RegressorDF,
+    Generic[T_FinalRegressorDF],
 ):
     """
     A data frame enabled pipeline with an optional preprocessing step and a
@@ -271,7 +297,9 @@ class RegressorPipelineDF(
 
 @inheritdoc(match="[see superclass]")
 class ClassifierPipelineDF(
-    LearnerPipelineDF[T_FinalClassifierDF], ClassifierDF, Generic[T_FinalClassifierDF]
+    SupervisedLearnerPipelineDF[T_FinalClassifierDF],
+    ClassifierDF,
+    Generic[T_FinalClassifierDF],
 ):
     """
     A data frame enabled pipeline with an optional preprocessing step and a
@@ -309,7 +337,7 @@ class ClassifierPipelineDF(
         return "classifier"
 
     @property
-    def classes_(self) -> Sequence[Any]:
+    def classes_(self) -> Union[npt.NDArray[Any], List[npt.NDArray[Any]]]:
         """[see superclass]"""
         return self.final_estimator.classes_
 
@@ -339,4 +367,63 @@ class ClassifierPipelineDF(
         )
 
 
+@inheritdoc(match="[see superclass]")
+class ClusterPipelineDF(
+    LearnerPipelineDF[T_FinalClusterDF],
+    ClusterDF,
+    Generic[T_FinalClusterDF],
+):
+    """
+    A data frame enabled pipeline with an optional preprocessing step and a
+    mandatory clustering step.
+    """
+
+    def __init__(
+        self,
+        *,
+        preprocessing: Optional[TransformerDF] = None,
+        clusterer: T_FinalClusterDF,
+    ) -> None:
+        """
+        :param preprocessing: the preprocessing step in the pipeline (default: ``None``)
+        :param clusterer: the clusterer used in the pipeline
+        :type clusterer: :class:`.ClusterDF`
+        """
+        super().__init__(preprocessing=preprocessing)
+
+        if not isinstance(clusterer, ClusterDF):
+            raise TypeError(
+                f"arg predictor expected to be a {ClusterDF.__name__} but is a "
+                f"{type(clusterer).__name__}"
+            )
+        self.clusterer = clusterer
+
+    @property
+    def final_estimator(self) -> T_FinalClusterDF:
+        """[see superclass]"""
+        return self.clusterer
+
+    @property
+    def final_estimator_name(self) -> str:
+        """[see superclass]"""
+        return "cluster"
+
+    @property
+    def labels_(self) -> pd.Series:
+        """[see superclass]"""
+        return self.final_estimator.labels_
+
+    # noinspection PyPep8Naming
+    def fit_predict(
+        self,
+        X: pd.DataFrame,
+        y: Optional[Union[pd.Series, pd.DataFrame]] = None,
+        **fit_predict_params: Any,
+    ) -> Union[pd.Series, pd.DataFrame]:
+        """[see superclass]"""
+        return self.clusterer.fit_predict(
+            self._pre_transform(X), y, **fit_predict_params
+        )
+
+
 __tracker.validate()
