@@ -8,7 +8,6 @@ from sklearn.pipeline import FeatureUnion, Pipeline
 
 from pytools.api import AllTracker
 
-from ..wrapper import make_df_estimator, make_df_transformer
 from .wrapper import FeatureUnionWrapperDF, PipelineWrapperDF
 
 log = logging.getLogger(__name__)
@@ -20,7 +19,7 @@ __all__ = ["PipelineDF", "FeatureUnionDF"]
 # Ensure all symbols introduced below are included in __all__
 #
 
-__tracker = AllTracker(globals(), allow_imported_definitions=True)
+__tracker = AllTracker(globals())
 
 
 #
@@ -28,8 +27,15 @@ __tracker = AllTracker(globals(), allow_imported_definitions=True)
 #
 
 
-PipelineDF = make_df_estimator(Pipeline, base_wrapper=PipelineWrapperDF)
-FeatureUnionDF = make_df_transformer(FeatureUnion, base_wrapper=FeatureUnionWrapperDF)
+class PipelineDF(PipelineWrapperDF, native=Pipeline):
+    """Stub for DF wrapper of class ``Pipeline``"""
+
+
+class FeatureUnionDF(
+    FeatureUnionWrapperDF,
+    native=FeatureUnion,
+):
+    """Stub for DF wrapper of class ``FeatureUnion``"""
 
 
 #
