@@ -1,19 +1,18 @@
 """
-Additional implementation of :mod:`sklearndf.regression` loaded
-from sklearn 1.0 onwards
+Core implementation of :mod:`sklearndf.regression` loaded
+from sklearn 0.22 onwards
 """
 import logging
 
-from sklearn.ensemble import HistGradientBoostingRegressor
-from sklearn.linear_model import QuantileRegressor
+from sklearn.ensemble import StackingRegressor
 
 from pytools.api import AllTracker
 
-from ..wrapper import RegressorWrapperDF
+from ..wrapper.stacking import StackingRegressorWrapperDF
 
 log = logging.getLogger(__name__)
 
-__all__ = ["HistGradientBoostingRegressorDF", "QuantileRegressorDF"]
+__all__ = ["StackingRegressorDF"]
 
 __imported_estimators = {name for name in globals().keys() if name.endswith("DF")}
 
@@ -26,27 +25,15 @@ __tracker = AllTracker(globals())
 
 
 #
-# ensemble
+# Class definitions
 #
 
 
-class HistGradientBoostingRegressorDF(
-    RegressorWrapperDF[HistGradientBoostingRegressor],
-    native=HistGradientBoostingRegressor,
+class StackingRegressorDF(
+    StackingRegressorWrapperDF[StackingRegressor],
+    native=StackingRegressor,
 ):
-    """Stub for DF wrapper of class ``HistGradientBoostingRegressor``"""
-
-
-#
-# linear model
-#
-
-
-class QuantileRegressorDF(
-    RegressorWrapperDF[QuantileRegressor],
-    native=QuantileRegressor,
-):
-    """Stub for DF wrapper of class ``QuantileRegressor``"""
+    """Stub for DF wrapper of class ``StackingRegressor``"""
 
 
 #
@@ -60,6 +47,7 @@ __tracker.validate()
 # validate that __all__ comprises all symbols ending in "DF", and no others
 #
 
+
 __estimators = {
     sym
     for sym in dir()
