@@ -1,3 +1,5 @@
+from typing import Type
+
 import numpy as np
 import pandas as pd
 import pytest
@@ -5,17 +7,25 @@ from sklearn.base import is_classifier
 from sklearn.ensemble import RandomForestClassifier
 from sklearn.preprocessing import OneHotEncoder
 
+from sklearndf import ClassifierDF
 from sklearndf.classification import RandomForestClassifierDF
+from sklearndf.classification.extra import LGBMClassifierDF
 from sklearndf.pipeline import ClassifierPipelineDF
 from test.sklearndf.pipeline import make_simple_transformer
 
 
+@pytest.mark.parametrize(  # type: ignore
+    argnames="classifier_df_cls",
+    argvalues=[RandomForestClassifierDF, LGBMClassifierDF],
+)
 def test_classification_pipeline_df(
-    iris_features: pd.DataFrame, iris_target_sr: pd.DataFrame
+    iris_features: pd.DataFrame,
+    iris_target_sr: pd.DataFrame,
+    classifier_df_cls: Type[ClassifierDF],
 ) -> None:
 
     cls_p_df = ClassifierPipelineDF(
-        classifier=RandomForestClassifierDF(),
+        classifier=classifier_df_cls(),
         preprocessing=make_simple_transformer(
             impute_median_columns=iris_features.select_dtypes(
                 include=np.number
